@@ -2,7 +2,17 @@ import importlib.metadata
 import inspect
 from collections.abc import Iterable
 from types import GenericAlias, UnionType
-from typing import Annotated, Any, Callable, ClassVar, Union, get_args, get_origin
+from typing import (
+    TYPE_CHECKING,
+    Annotated,
+    Any,
+    Callable,
+    ClassVar,
+    Mapping,
+    Union,
+    get_args,
+    get_origin,
+)
 
 from cached_classproperty import cached_classproperty
 from pydantic import BaseConfig, BaseModel, Extra, Field
@@ -18,7 +28,7 @@ PATCH_REQUEST_ATTR = "__patch_request__"
 
 
 def _resolve_annotation(annotation, attr: str) -> Any:
-    if inspect.isclass(annotation) and isinstance(annotation, ModelDuplicatorMeta):
+    if inspect.isclass(annotation) and isinstance(annotation, DualBaseModelMeta):
         return getattr(annotation, attr)
     elif isinstance(annotation, GenericAlias):
         return GenericAlias(
@@ -41,7 +51,7 @@ def _alter_attrs(attrs: dict[str, object], name: str, attr: str):
     attrs = attrs.copy()
     if "__qualname__" in attrs:
         attrs["__qualname__"] = name
-    if "__annotations__" in attrs:
+    if "__annotations__" in attrs and isinstance(attrs["__annotations__"], dict):
         annotations = attrs["__annotations__"].copy()
         for key, val in annotations.items():
             annotations[key] = _resolve_annotation(val, attr)
@@ -69,55 +79,89 @@ def _lazily_initalize_models(request_cls: type, own_attr_name: str, constructor:
 
 
 @dataclass_transform(kw_only_default=True, field_specifiers=(Field, FieldInfo))
-class ModelDuplicatorMeta(ModelMetaclass):
+class DualBaseModelMeta(ModelMetaclass):
     __request__: Self
     __response__: Self
     __patch_request__: Self
 
-    def __new__(mcls, name: str, bases: tuple[type], attrs: dict[str, object], **kwargs) -> Self:
-        new_class = type.__new__(mcls, name, bases, attrs)
-        if not bases or not any(isinstance(b, (ModelMetaclass, ModelDuplicatorMeta)) for b in bases):
+    def __new__(
+        cls,
+        name: str,
+        bases: tuple[type],
+        attrs: dict[str, object],
+        *,
+        request_suffix: str | None = None,
+        response_suffix: str | None = None,
+        patch_request_suffix: str | None = None,
+        **kwargs,
+    ) -> Self:
+        new_class = type.__new__(cls, name, bases, attrs)
+        if not bases or not any(isinstance(b, (ModelMetaclass, DualBaseModelMeta)) for b in bases):
             raise TypeError(
-                f"ModelDuplicatorMeta's instances must be created with a ConfigMixin base class or a BaseModel base class."
+                f"ModelDuplicatorMeta's instances must be created with a DualBaseModel base class or a BaseModel base class."
             )
-        # ConfigMixin case
+        # DualBaseModel case
         elif bases == (BaseModel,):
             if "__config__" not in kwargs:
                 raise TypeError(
-                    f"The first instance of {mcls.__name__} must pass a __config__ argument into the __new__ method."
+                    f"The first instance of {cls.__name__} must pass a __config__ argument into the __new__ method."
                 )
+            elif not inspect.isclass(kwargs["__config__"]):
+                raise TypeError("The __config__ argument must be a class.")
+            elif request_suffix is None or response_suffix is None or patch_request_suffix is None:
+                raise TypeError(
+                    "The first instance of DualBaseModel must pass suffixes for the request, response, and patch request models."
+                )
+            new_class._generate_base_alternative_classes(request_suffix, response_suffix, kwargs)
+        else:
+            request_suffix, response_suffix, patch_request_suffix = (
+                request_suffix or new_class.request_suffix,
+                response_suffix or new_class.response_suffix,
+                patch_request_suffix or new_class.patch_request_suffix,
+            )
+            new_class._generate_alternative_classes(
+                name, bases, attrs, request_suffix, response_suffix, patch_request_suffix, kwargs
+            )
 
-            __config__ = kwargs["__config__"]
+        new_class.__request__.request_suffix = request_suffix  # type: ignore
+        new_class.__request__.response_suffix = response_suffix  # type: ignore
+        new_class.__request__.patch_request_suffix = patch_request_suffix  # type: ignore
 
-            class BaseRequest(BaseModel):
-                class Config(__config__):
-                    extra = Extra.forbid
+        return new_class
 
-            class BaseResponse(BaseModel):
-                class Config(__config__):
-                    extra = Extra.ignore
+    def _generate_base_alternative_classes(self, request_suffix, response_suffix, kwargs):
+        class Config(kwargs["__config__"]):  # type: ignore
+            extra = Extra.forbid
 
-            type.__setattr__(new_class, "__request__", BaseRequest)
-            BaseRequest.__request__ = BaseRequest  # type: ignore
-            BaseRequest.__response__ = BaseResponse  # type: ignore
-            BaseRequest.__patch_request__ = BaseRequest  # type: ignore
-            return new_class
+        BaseRequest = ModelMetaclass(f"Base{request_suffix}", (BaseModel,), {"Config": Config})
 
+        class Config(kwargs["__config__"]):
+            extra = Extra.ignore
+
+        BaseResponse = ModelMetaclass(f"Base{response_suffix}", (BaseModel,), {"Config": Config})
+
+        type.__setattr__(self, "__request__", BaseRequest)
+        BaseRequest.__request__ = BaseRequest  # type: ignore
+        BaseRequest.__response__ = BaseResponse  # type: ignore
+        BaseRequest.__patch_request__ = BaseRequest  # type: ignore
+
+    def _generate_alternative_classes(
+        self, name, bases, attrs, request_suffix, response_suffix, patch_request_suffix, kwargs
+    ):
         request_bases = tuple(_resolve_annotation(b, REQUEST_ATTR) for b in bases)
-
         request_class = ModelMetaclass(
-            f"{name}Request",
+            name + request_suffix,
             request_bases,
-            _alter_attrs(attrs, f"{name}Request", REQUEST_ATTR),
+            _alter_attrs(attrs, name + request_suffix, REQUEST_ATTR),
             **kwargs,
         )
         request_class.__response__ = _lazily_initalize_models(
             request_class,
             RESPONSE_ATTR,
             lambda: ModelMetaclass(
-                f"{name}Response",
+                name + response_suffix,
                 tuple(_resolve_annotation(b, RESPONSE_ATTR) for b in bases),
-                _alter_attrs(attrs, f"{name}Response", RESPONSE_ATTR),
+                _alter_attrs(attrs, name + response_suffix, RESPONSE_ATTR),
                 **kwargs,
             ),
         )
@@ -125,20 +169,20 @@ class ModelDuplicatorMeta(ModelMetaclass):
             request_class,
             PATCH_REQUEST_ATTR,
             lambda: ModelMetaclass(
-                f"{name}PatchRequest",
+                name + patch_request_suffix,
                 tuple(_resolve_annotation(b, PATCH_REQUEST_ATTR) for b in bases),
-                _alter_attrs(attrs, f"{name}PatchRequest", PATCH_REQUEST_ATTR),
+                _alter_attrs(attrs, name + patch_request_suffix, PATCH_REQUEST_ATTR),
                 **kwargs,
             ),
         )
 
-        type.__setattr__(new_class, REQUEST_ATTR, request_class)
+        type.__setattr__(self, REQUEST_ATTR, request_class)
 
-        return new_class
+        return request_class
 
     def __getattribute__(self, attr: str):
-        # Note here that RESPONSE_ATTR goes into REQUEST_ATTR's __getattribute__ method
-        if attr in (REQUEST_ATTR, "__new__"):
+        # Note here that RESPONSE_ATTR and PATCH_REQUEST_ATTR goes into REQUEST_ATTR's __getattribute__ method
+        if attr in {REQUEST_ATTR, "__new__", "_generate_base_alternative_classes", "_generate_alternative_classes"}:
             return type.__getattribute__(self, attr)
         return getattr(type.__getattribute__(self, REQUEST_ATTR), attr)
 
@@ -149,7 +193,7 @@ class ModelDuplicatorMeta(ModelMetaclass):
         return set(super().__dir__()) | set(dir(getattr(self, REQUEST_ATTR)))
 
     def __eq__(self, __o: object) -> bool:
-        if isinstance(__o, ModelDuplicatorMeta):
+        if isinstance(__o, DualBaseModelMeta):
             return super().__eq__(__o)
         else:
             return self.__request__ == __o
@@ -164,28 +208,47 @@ class ModelDuplicatorMeta(ModelMetaclass):
         return type.__subclasscheck__(cls, subclass) or issubclass(subclass, cls.__request__)
 
 
-class ConfigMixin(BaseModel, metaclass=ModelDuplicatorMeta, __config__=BaseConfig):
-    __response__: ClassVar[type[Self]]
-    __request__: ClassVar[type[Self]]
-    __patch_request__: ClassVar[type[Self]]
-
-    def __new__(cls, *args, **kwargs) -> Self:
-        return cls.__request__(*args, **kwargs)
-
-    def __init_subclass__(cls, **kwargs) -> None:
-        return object.__init_subclass__()
-
-
-def generate_config_mixin(base_config) -> type[ConfigMixin]:
-    class ConfigMixin(BaseModel, metaclass=ModelDuplicatorMeta, __config__=base_config):
+def generate_dual_base_model(
+    base_config: type | Mapping = BaseConfig,
+    response_suffix="Response",
+    request_suffix="Request",
+    patch_request_suffix="PatchRequest",
+) -> "type[DualBaseModel]":
+    class DualBaseModel(
+        BaseModel,
+        metaclass=DualBaseModelMeta,
+        __config__=base_config,
+        request_suffix=request_suffix,
+        response_suffix=response_suffix,
+        patch_request_suffix=patch_request_suffix,
+    ):
         __response__: ClassVar[type[Self]]
         __request__: ClassVar[type[Self]]
         __patch_request__: ClassVar[type[Self]]
 
+        request_suffix: ClassVar[str]
+        response_sufx: ClassVar[str]
+        patch_request_suffix: ClassVar[str]
+
         def __new__(cls, *args, **kwargs):
             return cls.__request__(*args, **kwargs)
 
         def __init_subclass__(cls, **kwargs) -> None:
             return object.__init_subclass__()
 
-    return ConfigMixin
+    return DualBaseModel
+
+
+if TYPE_CHECKING:
+
+    class DualBaseModel(BaseModel, metaclass=DualBaseModelMeta, __config__=BaseConfig):
+        __request__: ClassVar[type[Self]]
+        __response__: ClassVar[type[Self]]
+        __patch_request__: ClassVar[type[Self]]
+
+        request_suffix: ClassVar[str]
+        response_sufx: ClassVar[str]
+        patch_request_suffix: ClassVar[str]
+
+else:
+    DualBaseModel = generate_dual_base_model()
